import { Request, Response } from 'express';
import db, { handlePrismaError, FTS_starChecker } from '../../../handlers/db.js'; // Database
import log from '../../../handlers/log.js';
import { parseDynamicSortBy } from '../../../helpers/prisma_helpers.js';
import { schema_get, schema_post, schema_patch, schema_del } from './alertContacts_schema.js';

// MARK: GET alertContact
async function get(req: Request, res: Response) {
	const { error, value } = schema_get.validate(req.query);
	if (error) {
		log.api?.debug('alertContact GET Error:', req.query, value, error.details[0].message);
		res.status(400).json({ status: 'ERROR', errorcode: 'VALIDATION_ERROR', message: error.details[0].message });
	} else {
		log.api?.debug('alertContact GET Success:', req.query, value);

		// Query with FullTextSearch
		if (value.search !== undefined || value.id !== undefined) {
			if (value.search !== undefined && FTS_starChecker(value.search)) {
				res.status(400).json({ status: 'ERROR', errorcode: 'VALIDATION_ERROR', message: 'Search query cannot contain only asterisk/s (*)' });
				return;
			}
			// with FullTextSearch
			if (!value.count) {
				// get all entrys
				log.api?.trace('get all entrys - with FullTextSearch');
				await db.alertContacts
					.findMany({
						where: {
							OR: [{ id: value.id }, { name: { search: value.search } }, { phone: { search: value.search } }, { comment: { search: value.search } }]
						},
						orderBy: parseDynamicSortBy(value.sort.toString(), value.order.toString()),
						skip: value.skip,
						take: value.take
					})
					.then((result) => {
						if (result.length !== 0) {
							res.status(200).json(result);
						} else {
							res.status(404).json({ status: 'ERROR', errorcode: 'NOT_FOUND', message: 'Could not find specified object' });
						}
					}).catch((err) => {
						handlePrismaError(err, res, 'GET alertContact');
					});
			} else {
				// count all entrys
				log.api?.trace('count all entrys - with FullTextSearch');
				await db.alertContacts
					.count({
						where: {
							OR: [{ id: value.id }, { name: { search: value.search } }, { phone: { search: value.search } }, { comment: { search: value.search } }]
						},
						orderBy: parseDynamicSortBy(value.sort.toString(), value.order.toString())
					})
					.then((result) => {
						res.status(200).json(result);
					}).catch((err) => {
						handlePrismaError(err, res, 'GET alertContact');
					});
			}
		} else {
			// without FullTextSearch
			if (!value.count) {
				// get all entrys
				log.api?.trace('get all entrys - without FullTextSearch');
				await db.alertContacts
					.findMany({ orderBy: parseDynamicSortBy(value.sort.toString(), value.order.toString()), take: value.take, skip: value.skip })
					.then((result) => {
						if (result.length !== 0) {
							res.status(200).json(result);
						} else {
							res.status(404).json({ status: 'ERROR', errorcode: 'NOT_FOUND', message: 'Could not find specified object' });
						}
					}).catch((err) => {
						handlePrismaError(err, res, 'GET alertContact');
					});
			} else {
				// count all entrys without FullTextSearch
				log.api?.trace('count all entrys - without FullTextSearch');
				await db.alertContacts.count().then((result) => {
					res.status(200).json(result);
				}).catch((err) => {
					handlePrismaError(err, res, 'GET alertContact');
				});
			}
		}
	}
}

// MARK: CREATE alertContact
async function post(req: Request, res: Response) {
	const { error, value } = schema_post.validate(req.body);
	if (error) {
		log.api?.debug('alertContact POST Error:', req.body, value, error.details[0].message);
		res.status(400).json({ status: 'ERROR', errorcode: 'VALIDATION_ERROR', message: error.details[0].message });
	} else {
		log.api?.debug('alertContact POST Success:', req.body, value);
		await db.alertContacts
			.create({
				data: {
					name: value.name,
					phone: value.phone,
					comment: value.comment
				},
				select: {
					id: true
				}
			})
			.then((result) => {
				res.status(201).json({ status: 'CREATED', message: 'Successfully created alertContact', id: result.id });
			}).catch((err) => {
				handlePrismaError(err, res, 'POST alertContact');
			});
	}
}

// MARK: UPDATE alertContact
async function patch(req: Request, res: Response) {
	const { error, value } = schema_patch.validate(req.body);
	if (error) {
		log.api?.debug('alertContact PATCH Error:', req.body, value, error.details[0].message);
		res.status(400).json({ status: 'ERROR', errorcode: 'VALIDATION_ERROR', message: error.details[0].message });
	} else {
		log.api?.debug('alertContact PATCH Success:', req.body, value);
		await db.alertContacts
			.update({
				where: {
					id: value.id
				},
				data: {
					name: value.name,
					phone: value.phone,
					comment: value.comment
				},
				select: {
					id: true
				}
			})
			.then((result) => {
				res.status(200).json({ status: 'UPDATED', message: 'Successfully updated alertContact', id: result.id });
			}).catch((err) => {
				handlePrismaError(err, res, 'PATCH alertContact');
			});
	}
}

// MARK: DELETE alertContact
async function del(req: Request, res: Response) {
	const { error, value } = schema_del.validate(req.body);
	if (error) {
		log.api?.debug('alertContact DELETE Error:', req.body, value, error.details[0].message);
		res.status(400).json({ status: 'ERROR', errorcode: 'VALIDATION_ERROR', message: error.details[0].message });
	} else {
		log.api?.debug('alertContact DELETE Success:', req.body, value);
		await db.alertContacts
			.delete({
				where: {
					id: value.id
				}
			})
			.then((result) => {
				res.status(200).json({ status: 'DELETED', message: 'Successfully deleted alertContact', id: result.id });
			}).catch((err) => {
				handlePrismaError(err, res, 'DEL alertContact');
			});
	}
}

export default { get, post, patch, del };
